Package: nanoparquet
Title: Read and Write 'Parquet' Files
Version: 0.5.1
Authors@R: c(
    person("Gábor", "Csárdi", role = c("aut", "cre"), email = "csardi.gabor@gmail.com"),
    person("Hannes", "Mühleisen", role = c("aut", "cph"), comment = c(ORCID = "0000-0001-8552-0029")),
    person("Google Inc.", role = "cph"),
    person("Apache Software Foundation", role = "cph"),
    person("Posit Software, PBC", role = "cph"),
    person("RAD Game Tools", role = "cph"),
    person("Valve Software", role = "cph"),
    person("Tenacious Software LLC", role = "cph"),
    person("Facebook, Inc.", role = "cph"))
Description: Self-sufficient reader and writer for flat 'Parquet' files.
    Can read most 'Parquet' data types. Can write many 'R' data types,
    including factors and temporal types. See docs for limitations.
Depends: R (>= 4.0.0)
License: MIT + file LICENSE
URL: https://github.com/r-lib/nanoparquet,
        https://nanoparquet.r-lib.org/
BugReports: https://github.com/r-lib/nanoparquet/issues
Encoding: UTF-8
Suggests: arrow, bit64, blob, DBI, duckdb (>= 1.4.0), hms, mockery,
        pillar, processx, rprojroot, spelling, testthat, tzdb, withr
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Config/Needs/website: tidyverse/tidytemplate, r-lib/pkgdown, dplyr, gt,
        gtExtras, knitr, nycflights13, prettyunits, quarto, rmarkdown,
        sessioninfo, svglite
Language: en-US
Biarch: true
Config/roxygen2/version: 7.3.3.9000
NeedsCompilation: yes
Packaged: 2026-04-20 10:55:25 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Hannes Mühleisen [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-8552-0029>),
  Google Inc. [cph],
  Apache Software Foundation [cph],
  Posit Software, PBC [cph],
  RAD Game Tools [cph],
  Valve Software [cph],
  Tenacious Software LLC [cph],
  Facebook, Inc. [cph]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-20 11:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:19:19 UTC; unix
