# Generated by roxygen2: do not edit by hand

S3method(pillar::obj_sum,nanoparquet_logical_type)
export(append_parquet)
export(infer_parquet_schema)
export(parquet_column_types)
export(parquet_info)
export(parquet_metadata)
export(parquet_options)
export(parquet_schema)
export(read_parquet)
export(read_parquet_info)
export(read_parquet_metadata)
export(read_parquet_schema)
export(write_parquet)
useDynLib(nanoparquet, .registration=TRUE)
