# as_count

    Code
      as_count(1:2)
    Condition
      Error in `as_count()`:
      ! x must be a count, i.e. an integer scalar
    Code
      as_count(0)
    Condition
      Error in `as_count()`:
      ! x must be a count, i.e. an integer scalar
    Code
      as_count(NA_real_)
    Condition
      Error in `as_count()`:
      ! x must be a count, i.e. an integer scalar
    Code
      as_count(-100)
    Condition
      Error in `as_count()`:
      ! x must be a count, i.e. an integer scalar
    Code
      as_count(-100L)
    Condition
      Error in `as_count()`:
      ! x must be a count, i.e. an integer scalar

