# obj_sum.nanoparquet_logical_type

    Code
      lapply(sch$logical_type, obj_sum.nanoparquet_logical_type)
    Output
      [[1]]
      [1] "INT(8, TRUE)"
      
      [[2]]
      [1] "DECIMAL(1, 3)"
      
      [[3]]
      [1] "TIME"
      
      [[4]]
      [1] "TIMESTAMP"
      

