# REQ PLAIN

    Code
      tapply(pgs$num_values, pgs$column + 1, sum)
    Output
          1     2     3     4 
      10000 10000 10000 10000 

---

    Code
      tapply(pgs$num_values, pgs$column + 1, sum)
    Output
          1     2     3     4 
      10000 10000 10000 10000 

# OPT PLAIN

    Code
      tapply(pgs$num_values, pgs$column + 1, sum)
    Output
          1     2     3     4 
      10000 10000 10000 10000 

---

    Code
      tapply(pgs$num_values, pgs$column + 1, sum)
    Output
          1     2     3     4 
      10000 10000 10000 10000 

